@@ -16,7 +16,7 @@ Web::DataService - a framework for building data service applications for the We
 
 =head1 VERSION
 
-Version 0.254
+Version 0.31
 
 =head1 SYNOPSIS
 
@@ -55,7 +55,9 @@ as Mojolicious and Catalyst soon.
 
 package Web::DataService;
 
-our $VERSION = '0.254';
+our $VERSION = '0.4';
+
+use feature qw(say);
 
 use Carp qw( carp croak confess );
 use Scalar::Util qw( reftype blessed weaken );
@@ -71,6 +73,7 @@ use Web::DataService::Render;
 use Web::DataService::Output;
 use Web::DataService::Execute;
 use Web::DataService::Document;
+use Web::DataService::Diagnostic;
 
 use Web::DataService::Request;
 use Web::DataService::IRequest;
@@ -84,12 +87,12 @@ with 'Web::DataService::Node', 'Web::DataService::Set',
      'Web::DataService::Format', 'Web::DataService::Vocabulary',
      'Web::DataService::Ruleset', 'Web::DataService::Render',
      'Web::DataService::Output', 'Web::DataService::Execute',
-     'Web::DataService::Document';
+     'Web::DataService::Document', 'Web::DataService::Diagnostic';
 
 
-our (@CARP_NOT) = qw(Web::DataService::Request Moo);
+our (@CARP_NOT) = qw(Web::DataService::Request Web::DataService::Node Moo);
 
-HTTP::Validate->VERSION(0.45);
+HTTP::Validate->VERSION(0.47);
 
 
 our @HTTP_METHOD_LIST = ('GET', 'HEAD', 'POST', 'PUT', 'DELETE');
@@ -129,7 +132,7 @@ my (@DI_KEYS) = qw(data_provider data_source data_license license_url
 
 # Execution modes
 
-our ($DEBUG, $ONE_REQUEST, $CHECK_LATER, $QUIET);
+our ($DEBUG, $ONE_REQUEST, $ONE_PROCESS, $CHECK_LATER, $QUIET, $DIAGNOSTIC);
 
 
 # Variables for keeping track of data service instances
@@ -178,8 +181,12 @@ has doc_index => ( is => 'lazy', builder => sub { $_[0]->_init_value('doc_index'
 
 has doc_template_dir => ( is => 'lazy', builder => sub { $_[0]->_init_value('doc_template_dir') } );
 
+has doc_compile_dir => ( is => 'lazy', builder => sub { $_[0]->_init_value('doc_compiled_dir') } );
+
 has output_template_dir => ( is => 'lazy', builder => sub { $_[0]->_init_value('output_template_dir') } );
 
+has output_compile_dir => ( is => 'lazy', builder => sub { $_[0]->_init_value('output_compiled_dir') } );
+
 has data_source => ( is => 'lazy', builder => sub { $_[0]->_init_value('data_source') } );
 
 has data_provider => ( is => 'lazy', builder => sub { $_[0]->_init_value('data_provider') } );
@@ -422,11 +429,13 @@ sub BUILD {
 	# exists and is readable.
 	
 	my $doc_dir = $self->doc_template_dir;
+	my $doc_comp = $self->doc_compile_dir;
 	my $output_dir = $self->output_template_dir;
+	my $output_comp = $self->output_compile_dir;
 	
 	unless ( defined $doc_dir )
 	{
-	    my $default = $ENV{PWD} . '/doc';
+	    my $default = 'doc';
 	    
 	    if ( -r $default )
 	    {
@@ -451,7 +460,7 @@ sub BUILD {
 	
 	if ( $doc_dir )
 	{
-	    $doc_dir = $ENV{PWD} . '/' . $doc_dir
+	    $doc_dir = './' . $doc_dir
 		unless $doc_dir =~ qr{ ^ / }xs;
 	    
 	    croak "the documentation template directory '$doc_dir' is not readable: $!\n"
@@ -460,7 +469,8 @@ sub BUILD {
 	    $self->{doc_template_dir} = $doc_dir;
 	    
 	    $self->{doc_engine} = 
-		$self->{templating_plugin}->new_engine($self, { template_dir => $doc_dir });
+		$self->{templating_plugin}->new_engine($self, { template_dir => $doc_dir,
+							        compile_dir => $doc_comp });
 	    
 	    # If the attributes doc_header, doc_footer, etc. were not set,
 	    # check for the existence of defaults.
@@ -479,7 +489,7 @@ sub BUILD {
     
 	if ( $output_dir )
 	{
-	    $output_dir = $ENV{PWD} . '/' . $output_dir
+	    $output_dir = './' . $output_dir
 		unless $output_dir =~ qr{ ^ / }xs;
 	    
 	    croak "the output template directory '$output_dir' is not readable: $!\n"
@@ -488,7 +498,8 @@ sub BUILD {
 	    $self->{output_template_dir} = $output_dir;
 	    
 	    $self->{output_engine} =
-		$self->{templating_plugin}->new_engine($self, { template_dir => $output_dir });
+		$self->{templating_plugin}->new_engine($self, { template_dir => $output_dir,
+							        compile_dir => $output_comp });
 	}
     }
     
@@ -910,6 +921,9 @@ sub select {
     {
 	my $path = $FOUNDATION->get_request_path($outer);
 	
+	# If one of the defined data service instances matches the path,
+	# return that.
+	
 	foreach my $ds ( @WDS_INSTANCES )
 	{
 	    if ( defined $ds->{path_re} && $path =~ $ds->{path_re} )
@@ -918,8 +932,15 @@ sub select {
 	    }
 	}
 	
-	# If none of the instances match this path, then throw a 404 (Not
-	# Found) exception.
+	# If this is an OPTIONS request for '*', then return the first defined
+	# instance.
+	
+	if ( $FOUNDATION->get_http_method eq 'OPTIONS' && ($path eq '*' || $path eq '/*') )
+	{
+	    return $WDS_INSTANCES[0];
+	}
+	
+	# Otherwise throw a 404 (Not Found) exception.
 	
 	my @prefixes = sort keys %PREFIX_MAP;
 	my $good_values = join(', ', map { "/$_" } @prefixes);
@@ -942,6 +963,9 @@ sub select {
 }
 
 
+# get_connection
+#
+# Call the backend plugin to get a database connection for use by this data service.
 
 sub get_connection {
     
@@ -954,6 +978,35 @@ sub get_connection {
 
 
 
+# set_mode ( mode... )
+#
+# Set one or more execution modes, which control how the data service processes requests and
+# generates output. These modes are set for the entire process, so if multiple data services are
+# defined they will share the same modes. All of these modes have been made available primarily
+# for debugging and testing purposes. The available modes are as follows:
+# 
+#    debug		Write debugging output to STDERR, including the text of any SQL
+#			statements issued to the database.
+#    
+#    quiet		Suppress all output to STDERR, even important warning messages.
+#			This is useful mainly during unit tests.
+#    
+#    one_request	This mode should be set when a request is specified via command-line
+#			arguments. It causes only those roles necessary for the specified
+#			request to be initialized, instead of all roles included in the data
+#			service configuration.
+#    
+#    diagnostic		This mode should be set when diagnostic output is requested via the
+#			the command line. It directs the diagnostic routines to be called
+#			instead of the usual request-execution routines.
+#    
+#    one_process	This does not have any effect on the Web::DataService code, but can
+#			be checked by data operation modules. If it is set, then only one
+#			process is assumed to be active at any one time. Consequently, certain
+#			operations that would otherwise use temporary tables can use permanent
+#			tables instead, which makes debugging easier.
+#    
+
 sub set_mode {
     
     my ($self, @modes) = @_;
@@ -965,6 +1018,11 @@ sub set_mode {
 	    $DEBUG = 1 unless $QUIET || $ENV{WDS_QUIET};
 	}
 	
+	elsif ( $mode eq 'one_process' )
+	{
+	    $ONE_PROCESS = 1;
+	}
+	
 	elsif ( $mode eq 'one_request' )
 	{
 	    $ONE_REQUEST = 1;
@@ -980,10 +1038,19 @@ sub set_mode {
 	    $QUIET = 1;
 	    $DEBUG = 0;
 	}
+	
+	elsif ( $mode eq 'diagnostic' )
+	{
+	    $DIAGNOSTIC = 1;
+	}
     }
 }
 
 
+# is_mode ( mode )
+#
+# Returns true if the specified mode has been enabled, false otherwise.
+
 sub is_mode {
 
     my ($self, $mode) = @_;
@@ -992,10 +1059,49 @@ sub is_mode {
     return 1 if $mode eq 'one_request' && $ONE_REQUEST;
     return 1 if $mode eq 'late_path_check' && $CHECK_LATER;
     return 1 if $mode eq 'quiet' && $QUIET;
+    return 1 if $mode eq 'diagnostic' && $DIAGNOSTIC;
     return;
 }
 
 
+# Specifically return true if 'debug' mode is enabled, false otherwise.
+
+sub debug_mode {
+
+    my ($self) = @_;
+    
+    return $DEBUG;
+}
+
+
+sub debug {
+
+    my ($self) = @_;
+    
+    return $DEBUG;
+}
+
+
+# Output a line of debugging information, if 'debug' mode is enabled.
+
+sub debug_line {
+    
+    my ($self, $msg) = @_;
+    
+    print STDERR "$msg\n" if $DEBUG;
+}
+
+
+sub debug_list {
+    
+    return unless $DEBUG;
+    
+    my ($self, @msgs) = @_;
+    
+    print STDERR "$_\n\n" foreach @msgs;
+}
+
+
 # generate_site_url ( attrs )
 # 
 # Generate a URL according to the specified attributes:
@@ -1391,9 +1497,9 @@ before using it as a secondary role for '$primary_role'"
     }
     
     my $string =  " package $request_class;
-			use Try::Tiny;
-			use Scalar::Util qw(reftype);
-			use Carp qw(carp croak);
+			# use Try::Tiny;
+			# use Scalar::Util qw(reftype);
+			# use Carp qw(carp croak);
 			use Moo;
 			use namespace::clean;
 			
@@ -1405,6 +1511,20 @@ before using it as a secondary role for '$primary_role'"
     
     my $result = eval $string;
     
+    if ( $@ )
+    {
+	if ( $@ =~ qr{method name conflict.*the method '(.*?)'} )
+	{
+	    my $method = $1;
+	    croak "The method name '$method' in $primary_role conflicts with the same name in Web::DataService::IRequest; you must choose another name for this subroutine";
+	}
+	
+	else
+	{
+	    croak "$@";
+	}
+    }
+    
     # Now initialize the primary role, unless of course it has already been
     # initialized.  This will also cause any uninitialized secondary roles to
     # be initialized.
@@ -1640,13 +1760,6 @@ sub contact_info {
 # }
 
 
-sub debug {
-
-    my ($self) = @_;
-    
-    return $DEBUG || $self->{DEBUG};
-}
-
 =head1 MORE DOCUMENTATION
 
 This documentation describes the methods of class Web::DataService.  For
@@ -1678,6 +1791,10 @@ service element.
 
 An overview of the elements available for use in documentation templates.
 
+=item L<Web::DataService::Debugging>
+
+Features that are available for debugging your application.
+
 =back
 
 =head1 METHODS
@@ -1851,14 +1968,15 @@ clients use.  This may be different from the internal name by which this
 parameter is known, but will always be a true value.  Returns false if this
 parameter is not enabled.
 
-=head3 generate_site_url
+=head3 generate_site_url ( attrs )
 
 This method is called by the
 L<generate_url|Web::DataService::Request/"generate_url ( attrs )"> method of
 L<Web::DataService::Request>.  You should be aware that if you call it outside
 of the context of a request it will not be able to generate absolute URLs.  In
 most applications, you will never need to call this directly and can instead
-use the latter method.
+use the latter method. The argument must be a hash reference, and the accepted
+keys are listed in the documentation for the latter method.
 
 =head3 get_connection
 
@@ -1868,6 +1986,58 @@ initialization process requires communication with the backend.  You are not
 required to use this mechanism, and may connect to the backend in any way you
 choose.
 
+=head3 set_mode
+
+Turn on one of the following modes, which are provided primarily for testing and debugging
+purposes. These modes are set for the entire process, not for a single data service.
+
+=over
+
+=item debug
+
+Debugging statements will be written to STDERR. Any data service operation code that you write
+should check for this mode being set, and if it is then call C<debug_line> to write out
+appropriate debugging information including the text of any SQL statements issued to the database.
+
+=item quiet
+
+All output to STDERR is suppressed, even important warning messages. This is useful mainly during
+unit tests, where error conditions are explicitly generated to check that the proper response
+occurs. 
+
+=item one_request
+
+This mode should be set when a request is specified via command-line arguments. It causes only
+those roles necessary for the specified request to be initialized, instead of all roles included
+in the data service configuration.
+
+=item diagnostic
+
+This mode should be set when diagnostic output is requested via the command line. It directs the
+diagnostic routines to be called using the information specified on the command line, rather than
+the usual request-handling routines.
+
+=item one_process
+
+This mode should be set when a single data service process is run for debugging purposes. It has
+no effect on the Web::DataService code, but data operation code can check for it and use permanent
+tables rather than temporary ones, which can make debugging easier.
+
+=back
+
+=head3 is_mode ( mode )
+
+Returns true if the specified mode has been enabled for this data service process (not just for
+this data service). Returns false otherwise.
+
+=head3 debug
+
+Returns true if 'debug' mode has been enabled for this process (not just for this data service).
+
+=head3 debug_line ( text )
+
+If 'debug' mode is enabled, prints out the specified text followed by a newline to STDERR.
+
 =head3 accessor methods
 
 Each of the data service 
